// MISTY MOUNTAIN

// Main entry point for random map script
void main(void)
{
 rmCreateTrigger("night");
   rmCreateTrigger("Day");

   rmSwitchToTrigger(rmTriggerID("night"));
   rmSetTriggerActive(true);
   rmAddTriggerCondition("Timer");
   rmSetTriggerConditionParamInt("Param1", 120);
   rmAddTriggerEffect("Set Lighting");
   rmSetTriggerEffectParam("SetName", "night");
   rmSetTriggerEffectParamInt("FadeTime", 60);
   rmAddTriggerEffect("Fire Event");
   rmSetTriggerEffectParamInt("EventID", rmTriggerID("Day"));

   rmSwitchToTrigger(rmTriggerID("Day"));
   rmSetTriggerActive(false);
   rmAddTriggerCondition("Timer");
   rmSetTriggerConditionParamInt("Param1", 240);
   rmAddTriggerEffect("Set Lighting");
   rmSetTriggerEffectParam("SetName", "Default");
   rmSetTriggerEffectParamInt("FadeTime", 60);

   // Text
   rmSetStatusText("",0.01);

   // Set size.
   int playerTiles=20000;
   if(cMapSize == 1)
   {
      playerTiles = 23000;
      rmEchoInfo("Large map");
   }
   int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);

   // Init map.
   rmTerrainInitialize("grassdirt25");
   rmSetLightingSet("default");
 
    // Define some classes.
   int classPlayer=rmDefineClass("player");
   int classPlayerCore=rmDefineClass("player core");
   int classcliff=rmDefineClass("cliff");
   int classpatch=rmDefineClass("patch");
   rmDefineClass("corner");
   rmDefineClass("starting settlement");




   // -------------Define constraints
   
   // Create a edge of map constraint.
   int edgeConstraint=rmCreateBoxConstraint("edge of map", rmXTilesToFraction(0), rmZTilesToFraction(0), 1.0-rmXTilesToFraction(0), 1.0-rmZTilesToFraction(0));
   int mistCenterConstraint=rmCreateBoxConstraint("mist stay in center", 0.25, 0.15, 0.75, 0.85);
   int tamariskCenterConstraint=rmCreateBoxConstraint("tamarisk stay in center", 0.25, 0.15, 0.75, 0.85);

   
// corner constraint.
   int cornerOverlapConstraint=rmCreateClassDistanceConstraint("don't overlap corner", rmClassID("corner"), 2.0);
   int cornerConstraint=rmCreateClassDistanceConstraint("stay away from corner", rmClassID("corner"), 15.0);

   // Settlement constraints
   int shortAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by short distance", "AbstractSettlement", 20.0);
   int farAvoidSettlement=rmCreateTypeDistanceConstraint("TCs avoid TCs by long distance", "AbstractSettlement", 40.0);
   int farStartingSettleConstraint=rmCreateClassDistanceConstraint("objects avoid player TCs", rmClassID("starting settlement"), 60.0);
   int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, 25);
   int settleConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, 60);
       
   // Tower constraint.
   int avoidTower=rmCreateTypeDistanceConstraint("towers avoid towers", "tower", 25.0);
   int avoidTower2=rmCreateTypeDistanceConstraint("objects avoid towers", "tower", 25.0);

   // Gold
   int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 10.0);
   int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 10.0);

   // Food
   int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 20.0);
   int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 20.0);
   int avoidFood=rmCreateTypeDistanceConstraint("avoid other food sources", "food", 6.0);

   // Avoid impassable land
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "land", false, 10.0);
/*   int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land short", "land", false, 5.0); */
   int cliffConstraint=rmCreateClassDistanceConstraint("cliff v gorge", rmClassID("cliff"), 30.0);
   int shortCliffConstraint=rmCreateClassDistanceConstraint("elev v gorge", rmClassID("cliff"), 10.0);
   int oceanConstraint=rmCreateClassDistanceConstraint("gorge vs ocean", rmClassID("ocean"), 25.0);
   int forestOceanConstraint=rmCreateClassDistanceConstraint("forest vs ocean", rmClassID("ocean"), 10.0);
   int smallOceanConstraint=rmCreateClassDistanceConstraint("terrain vs ocean", rmClassID("ocean"), 10.0);
   int patchConstraint=rmCreateClassDistanceConstraint("patch v patch", rmClassID("patch"), 10.0);

  
   // -------------Define objects
   // Close Objects

   int startingSettlementID=rmCreateObjectDef("starting settlement");
   rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
   rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
   rmSetObjectDefMinDistance(startingSettlementID, 0.0);
   rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

   // towers avoid other towers
   int startingTowerID=rmCreateObjectDef("Starting tower");
   rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
   rmSetObjectDefMinDistance(startingTowerID, 22.0);
   rmSetObjectDefMaxDistance(startingTowerID, 28.0);
   rmAddObjectDefConstraint(startingTowerID, avoidTower);
   rmAddObjectDefConstraint(startingTowerID, avoidImpassableLand);
   
   // gold avoids gold
   int startingGoldID=rmCreateObjectDef("Starting gold");
   rmAddObjectDefItem(startingGoldID, "Gold mine", 2, 2.0);
   rmSetObjectDefMinDistance(startingGoldID, 15.0);
   rmSetObjectDefMaxDistance(startingGoldID, 50.0);
   rmAddObjectDefConstraint(startingGoldID, avoidGold);
   rmAddObjectDefConstraint(startingGoldID, avoidImpassableLand);

   // goats
   
   int GoatNum=rmRandInt(2, 5);
   int closegoatsID=rmCreateObjectDef("close goats");
   rmAddObjectDefItem(closegoatsID, "goat", GoatNum, 2.0);
   rmSetObjectDefMinDistance(closegoatsID, 25.0);
   rmSetObjectDefMaxDistance(closegoatsID, 30.0);
   rmAddObjectDefConstraint(closegoatsID, avoidFood);
   rmAddObjectDefConstraint(closegoatsID, avoidImpassableLand);
   
   int numChicken = 0;
   int numBerry = 0;
   float berryChance = rmRandFloat(0,1);
   if(berryChance < 0.25)
   {
      numChicken = 4;
      numBerry = 3;
   }
   else if(berryChance < 0.75)
   {
      numChicken = 8;
      numBerry = 6;
   }
   else
   {
      numChicken = 12;
      numBerry = 9;
   }
      
   int closeChickensID=rmCreateObjectDef("close Chickens");
   rmAddObjectDefItem(closeChickensID, "chicken", numChicken, 2.0);
   rmSetObjectDefMinDistance(closeChickensID, 20.0);
   rmSetObjectDefMaxDistance(closeChickensID, 25.0);
   rmAddObjectDefConstraint(closeChickensID, avoidFood);
   
   int closeBerriesID=rmCreateObjectDef("close berries");
   rmAddObjectDefItem(closeBerriesID, "berry bush", numBerry, 2.0);
   rmSetObjectDefMinDistance(closeBerriesID, 20.0);
   rmSetObjectDefMaxDistance(closeBerriesID, 25.0);
   rmAddObjectDefConstraint(closeBerriesID, avoidFood);

   int closeBoarID=rmCreateObjectDef("close Boar");
   float boarNumber=rmRandFloat(0, 1);
   if(boarNumber<0.3)
      rmAddObjectDefItem(closeBoarID, "boar", 3, 4.0);
   else if(boarNumber<0.9)
      rmAddObjectDefItem(closeBoarID, "boar", 4, 4.0);
   else
      rmAddObjectDefItem(closeBoarID, "boar", 1, 4.0);
   rmSetObjectDefMinDistance(closeBoarID, 30.0);
   rmSetObjectDefMaxDistance(closeBoarID, 50.0);

   int stragglerTreeID=rmCreateObjectDef("straggler tree");
   rmAddObjectDefItem(stragglerTreeID, "oak tree", 2, 2.0);
   rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
   rmSetObjectDefMaxDistance(stragglerTreeID, 600.0);

// Medium Objects

   // gold avoids gold and Settlements
   int mediumGoldID=rmCreateObjectDef("medium gold");
   rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(mediumGoldID, 150.0);
   rmSetObjectDefMaxDistance(mediumGoldID, 250.0);
   rmAddObjectDefConstraint(mediumGoldID, shortAvoidGold);
   rmAddObjectDefConstraint(mediumGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(mediumGoldID, farStartingSettleConstraint);

   // Text
   rmSetStatusText("",0.10);

   int mediumGoatsID=rmCreateObjectDef("medium goats");
   rmAddObjectDefItem(mediumGoatsID, "goat", 2, 4.0);
   rmSetObjectDefMinDistance(mediumGoatsID, 50.0);
   rmSetObjectDefMaxDistance(mediumGoatsID, 80.0);
   rmAddObjectDefConstraint(mediumGoatsID, avoidHerdable);
   rmAddObjectDefConstraint(mediumGoatsID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumGoatsID, farStartingSettleConstraint);

   // Far Objects
        
   // gold avoids gold, Settlements and TCs
   int farGoldID=rmCreateObjectDef("far gold");
   rmAddObjectDefItem(farGoldID, "Gold mine", 1, 1000.0);
   rmSetObjectDefMinDistance(farGoldID, 350.0);
   rmSetObjectDefMaxDistance(farGoldID, 1000.0); 
   rmAddObjectDefConstraint(farGoldID, avoidGold);
   rmAddObjectDefConstraint(farGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(farGoldID, farStartingSettleConstraint);

   // goats avoid TCs and other herds, since this map places a lot of goats
   int farGoatsID=rmCreateObjectDef("far goats");
   rmAddObjectDefItem(farGoatsID, "goat", 4, 4.0);
   rmSetObjectDefMinDistance(farGoatsID, 80.0);
   rmSetObjectDefMaxDistance(farGoatsID, 950.0);
   rmAddObjectDefConstraint(farGoatsID, avoidHerdable);
   rmAddObjectDefConstraint(farGoatsID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farGoatsID, edgeConstraint);
   
   // pick lions or bears as predators
   // avoid TCs
   int farPredatorID=rmCreateObjectDef("far predator");
   float predatorSpecies=rmRandFloat(0, 1);
   if(predatorSpecies<0.5)   
      rmAddObjectDefItem(farPredatorID, "bear", 2, 3.0);
   else
      rmAddObjectDefItem(farPredatorID, "wolf", 4, 4.0);
   rmSetObjectDefMinDistance(farPredatorID, 50.0);
   rmSetObjectDefMaxDistance(farPredatorID, 900.0);
   rmAddObjectDefConstraint(farPredatorID, avoidPredator);
   rmAddObjectDefConstraint(farPredatorID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farPredatorID, avoidImpassableLand);
   
   // Berries avoid TCs  
   int farBerriesID=rmCreateObjectDef("far berries");
   rmAddObjectDefItem(farBerriesID, "berry bush", 6, 4.0);
   rmSetObjectDefMinDistance(farBerriesID, 0.0);
   rmSetObjectDefMaxDistance(farBerriesID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(farBerriesID, avoidImpassableLand);
   rmAddObjectDefConstraint(farBerriesID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farBerriesID, edgeConstraint);
   
   // This map will either use boar or deer as the extra huntable food.
   int classBonusHuntable=rmDefineClass("bonus huntable");
   int avoidBonusHuntable=rmCreateClassDistanceConstraint("avoid bonus huntable", classBonusHuntable, 40.0);
   int avoidHuntable=rmCreateTypeDistanceConstraint("avoid huntable", "huntable", 20.0);

   // hunted avoids hunted and TCs
   int bonusHuntableID=rmCreateObjectDef("bonus huntable");
   float bonusChance=rmRandFloat(0, 1);
   if(bonusChance<0.5)   
      rmAddObjectDefItem(bonusHuntableID, "deer", 8, 4.0);
   else
      rmAddObjectDefItem(bonusHuntableID, "deer", 10, 4.0);
   rmSetObjectDefMinDistance(bonusHuntableID, 0.0);
   rmSetObjectDefMaxDistance(bonusHuntableID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(bonusHuntableID, avoidBonusHuntable);
   rmAddObjectDefConstraint(bonusHuntableID, avoidHuntable);
   rmAddObjectDefToClass(bonusHuntableID, classBonusHuntable);
   rmAddObjectDefConstraint(bonusHuntableID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(bonusHuntableID, avoidImpassableLand);

   int randomTreeID=rmCreateObjectDef("random tree");
   rmAddObjectDefItem(randomTreeID, "oak tree", 12, 4.0);
   rmSetObjectDefMinDistance(randomTreeID, 0.0);
   rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
   rmAddObjectDefConstraint(randomTreeID, shortAvoidSettlement);
   rmAddObjectDefConstraint(randomTreeID, avoidImpassableLand);

   // Relics avoid TCs
   int relicID=rmCreateObjectDef("relic");
   rmAddObjectDefItem(relicID, "relic", 3, 200.0);
   rmSetObjectDefMinDistance(relicID, 60.0);
   rmSetObjectDefMaxDistance(relicID, 800.0);
   rmAddObjectDefConstraint(relicID, edgeConstraint);
   rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 70.0));
   rmAddObjectDefConstraint(relicID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(relicID, avoidImpassableLand);


   // -------------Done defining objects

   // Cheesy "circular" placement of players.


   if(cNumberNonGaiaPlayers < 3)
         rmPlacePlayersLine(0.12, 0.5, 0.88, 0.5);
   else if(cNumberTeams < 3)
      {
         rmSetPlacementTeam(0);
         rmPlacePlayersLine(0.12, 0.20, 0.12, 0.80, 20, 10); /* x z x2 z2 */
         rmSetPlacementTeam(1);
         rmPlacePlayersLine(0.88, 0.20, 0.88, 0.80, 20, 10);
      } 
   else    
      {
      rmPlacePlayersCircular(0.3, 0.35, rmDegreesToRadians(5.0));
      }


   for(i=1; <cNumberPlayers)
   {
      // Create the area.
      int id=rmCreateArea("Player core"+i);

      // Set the size.
      rmSetAreaSize(id, rmAreaTilesToFraction(80), rmAreaTilesToFraction(100));

      rmAddAreaToClass(id, classPlayerCore);

      rmSetAreaCoherence(id, 1.0);

      // Set the location.
      rmSetAreaLocPlayer(id, i);

      // Build it.
      rmBuildArea(id);
   }


   // Text
   rmSetStatusText("",0.20);

   int failCount=0;

  
   rmSetStatusText("",0.30);

   // Set up player areas.
   float playerFraction=rmAreaTilesToFraction(500);
   for(i=1; <cNumberPlayers)
   {
      // Create the area.
      id=rmCreateArea("Player"+i);
      // Assign to the player.
      rmSetPlayerArea(i, id);
      // Set the size.
      rmSetAreaSize(id, 0.9*playerFraction, 1.1*playerFraction);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 1);
      rmSetAreaMinBlobDistance(id, 0.0);
      rmSetAreaMaxBlobDistance(id, 15.0);
      rmSetAreaCoherence(id, 0.8);
      rmAddAreaConstraint(id, edgeConstraint);
      rmSetAreaTerrainType(id, "greekroada");
      rmAddAreaTerrainLayer(id, "greekroada", 0, 3);

      // Set the location.
      rmSetAreaLocPlayer(id, i);
      // Set type.
   }


   // Build the areas.
   rmBuildAllAreas();

   // Map beautification sub area.

   int goldArea=rmCreateArea("here is gold");
   rmSetAreaSize(goldArea, 1.0, 1.0);
   rmSetAreaWarnFailure(goldArea, false);

   rmBuildArea(goldArea);

   int patch = 0;
   int stayInPatch=rmCreateEdgeDistanceConstraint("stay in patch", patch, 4.0);
   for(i=1; <cNumberPlayers*10) 
   {
      patch=rmCreateArea("patch"+i);
      rmSetAreaSize(patch, rmAreaTilesToFraction(100), rmAreaTilesToFraction(200));
      rmSetAreaWarnFailure(patch, false); 
      rmSetAreaTerrainType(patch, "grassdirt75");
      rmAddAreaTerrainLayer(patch, "shorelineaegeanb", 1, 2);
      rmAddAreaTerrainLayer(patch, "grassdirt50", 0, 1);
      rmSetAreaMinBlobs(patch, 15);
      rmSetAreaMaxBlobs(patch, 25);
      rmSetAreaMinBlobDistance(patch, 16.0);
      rmSetAreaMaxBlobDistance(patch, 900.0);
      rmAddAreaToClass(patch, classpatch);
      rmAddAreaConstraint(patch, patchConstraint); 
      rmAddAreaConstraint(patch, playerConstraint);
      rmSetAreaCoherence(patch, 0.0);
      rmSetAreaSmoothDistance(patch, 8);

      if(rmBuildArea(patch)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;

      int snowRockID=rmCreateObjectDef("snowRock"+i);
      rmAddObjectDefItem(snowRockID, "rock limestone sprite", rmRandFloat(0,3), 2.0);
      rmSetObjectDefMinDistance(snowRockID, 0.0);
      rmSetObjectDefMaxDistance(snowRockID, 0.0);
      rmAddObjectDefConstraint(snowRockID, stayInPatch);
      rmPlaceObjectDefInArea(snowRockID, 0, rmAreaID("patch"+i), 1);
   }

   for(i=1; <cNumberPlayers*20) 
   {
      int patch2=rmCreateArea("2nd patch"+i);
      rmSetAreaSize(patch2, rmAreaTilesToFraction(120), rmAreaTilesToFraction(150));
      rmSetAreaWarnFailure(patch2, false); 
      rmSetAreaTerrainType(patch2, "cliffgreekb");
      rmSetAreaMinBlobs(patch2, 10);
      rmSetAreaMaxBlobs(patch2, 15);
      rmSetAreaMinBlobDistance(patch2, 16.0);
      rmSetAreaMaxBlobDistance(patch2, 700.0);
      rmAddAreaConstraint(patch2, playerConstraint);
      rmAddAreaToClass(patch2, classpatch);
      rmAddAreaConstraint(patch2, patchConstraint); 
      rmSetAreaCoherence(patch2, 0.0);
      rmSetAreaSmoothDistance(patch2, 8);

      if(rmBuildArea(patch2)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
   }
for(i=1; <cNumberPlayers*20) 
   {
      int patch3=rmCreateArea("3nd patch"+i);
      rmSetAreaSize(patch3, rmAreaTilesToFraction(180), rmAreaTilesToFraction(210));
      rmSetAreaWarnFailure(patch3, false); 
      rmSetAreaTerrainType(patch3, "rivergrassyc");
      rmSetAreaMinBlobs(patch3, 6);
      rmSetAreaMaxBlobs(patch3, 10);
      rmSetAreaMinBlobDistance(patch3, 16.0);
      rmSetAreaMaxBlobDistance(patch3, 500.0);
      rmAddAreaConstraint(patch3, playerConstraint);
      rmAddAreaToClass(patch3, classpatch);
      rmAddAreaConstraint(patch3, patchConstraint); 
      rmSetAreaCoherence(patch3, 0.1);
      rmSetAreaSmoothDistance(patch3, 8);

      if(rmBuildArea(patch3)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
      }


   // Make gorges

   int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 20.0);
   int gorgeID=rmCreateArea("gorge 1");
      rmSetAreaWarnFailure(gorgeID, false); 
   rmSetAreaSize(gorgeID, rmAreaTilesToFraction(4900), rmAreaTilesToFraction(5100));
   rmSetAreaCliffType(gorgeID, "greek");
   rmAddAreaConstraint(gorgeID, cliffConstraint);
   rmAddAreaConstraint(gorgeID, avoidBuildings);
   rmAddAreaConstraint(gorgeID, oceanConstraint); 
   rmAddAreaToClass(gorgeID, classcliff);
   rmSetAreaMinBlobs(gorgeID, 15);
   rmSetAreaMaxBlobs(gorgeID, 18);
   rmSetAreaLocation(gorgeID, 0.5, 0.5);
   rmAddAreaInfluenceSegment(gorgeID, 0.4, 0.25, 0.4, 0.75); 
   rmSetAreaCliffEdge(gorgeID, 10, 0.08, 0.1, 1.0, 2);
   rmSetAreaCliffPainting(gorgeID, false, true, true, 1.5, true);
   rmSetAreaCliffHeight(gorgeID, -5, 1.0, 1.0);
   rmSetAreaMinBlobDistance(gorgeID, 10.0);
   rmSetAreaMaxBlobDistance(gorgeID, 300.0);
   rmSetAreaCoherence(gorgeID, 0.0);
   rmSetAreaSmoothDistance(gorgeID, 10);
   rmSetAreaCliffHeight(gorgeID, -5, 1.0, 1.0);
   rmSetAreaHeightBlend(gorgeID, 2);
   if(cNumberTeams < 3)
      {
      rmBuildArea(gorgeID);
      }

   int gorge2ID=rmCreateArea("gorge 2");
      rmSetAreaWarnFailure(gorge2ID, false); 
   rmSetAreaSize(gorge2ID, rmAreaTilesToFraction(5000), rmAreaTilesToFraction(5400));
   rmSetAreaCliffType(gorge2ID, "greek");
   rmAddAreaConstraint(gorge2ID, avoidBuildings);
   rmAddAreaConstraint(gorge2ID, oceanConstraint); 
   rmAddAreaToClass(gorge2ID, classcliff);
   rmSetAreaMinBlobs(gorge2ID, 35);
   rmSetAreaMaxBlobs(gorge2ID, 36);
   rmSetAreaLocation(gorge2ID, 0.5, 0.5);
   rmAddAreaInfluenceSegment(gorge2ID, 0.6, 0.25, 0.6, 0.75); 
   rmSetAreaCliffEdge(gorge2ID, 10, 0.09, 0.1, 1.0, 2);
   rmSetAreaCliffPainting(gorge2ID, false, true, true, 1.0, true);
   rmSetAreaCliffHeight(gorge2ID, -4, 1.0, 1.0);
   rmSetAreaMinBlobDistance(gorge2ID, 20.0);
   rmSetAreaMaxBlobDistance(gorge2ID, 200.0);
   rmSetAreaCoherence(gorge2ID, 0.0);
   rmSetAreaSmoothDistance(gorge2ID, 10);
   rmSetAreaCliffHeight(gorge2ID, -5, 1.0, 0.5);
   rmSetAreaHeightBlend(gorge2ID, 3);
   if(cNumberTeams < 3)
      {
      rmBuildArea(gorge2ID);
      }
   
   for(i=0; <8)
   {
      int cliffID=rmCreateArea("cliff"+i);
      rmSetAreaWarnFailure(cliffID, false);
      rmSetAreaSize(cliffID, rmAreaTilesToFraction(5200), rmAreaTilesToFraction(5500));
      rmSetAreaCliffType(cliffID, "greek");
      rmAddAreaConstraint(cliffID, playerConstraint);
      rmAddAreaConstraint(cliffID, avoidBuildings);
      rmAddAreaConstraint(cliffID, oceanConstraint); 
      rmAddAreaToClass(cliffID, classcliff);
      rmSetAreaMinBlobs(cliffID, 35);
      rmSetAreaMaxBlobs(cliffID, 35);
      // AOK style
      rmSetAreaCliffEdge(cliffID, 10, 0.09, 0.1, 1.0, 2);
      rmSetAreaCliffPainting(cliffID, false, true, true, 1.0);
      rmSetAreaCliffHeight(cliffID, 5, 0.5, 0.5);
      rmSetAreaMinBlobDistance(cliffID, 10.0);
      rmSetAreaMaxBlobDistance(cliffID, 100.0);
      rmSetAreaCoherence(cliffID, 0.0);
      rmSetAreaSmoothDistance(cliffID, 10);
      rmSetAreaCliffHeight(cliffID, 5, 1.0, 0.5);
      rmSetAreaHeightBlend(cliffID, 3); 
      rmBuildArea(cliffID); 
   }

   // Elev.
   int numTries=20*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <numTries)
   {
      int elevID=rmCreateArea("elev"+i);
      rmSetAreaSize(elevID, rmAreaTilesToFraction(1000), rmAreaTilesToFraction(1150));
      rmSetAreaLocation(elevID, rmRandFloat(0.0, 1.0), rmRandFloat(0.0, 1.0));
      rmSetAreaWarnFailure(elevID, false);
      rmAddAreaConstraint(elevID, cornerConstraint);
      rmAddAreaConstraint(elevID, avoidBuildings);
      rmAddAreaConstraint(elevID, shortCliffConstraint);
      rmAddAreaConstraint(elevID, playerConstraint); 
      if(rmRandFloat(0.0, 1.0)<0.5)
         rmSetAreaTerrainType(elevID, "cliffgreekb");
      rmSetAreaBaseHeight(elevID, rmRandFloat(2.0, 4.0));
      rmSetAreaMinBlobs(elevID, 20);
      rmSetAreaMaxBlobs(elevID, 25);
      rmSetAreaMinBlobDistance(elevID, 10.0);
      rmSetAreaMaxBlobDistance(elevID, 100.0);
      rmSetAreaCoherence(elevID, 0.0);

      if(rmBuildArea(elevID)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
   }

   // Slight Elevation
   numTries=35*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <numTries)
   {
      elevID=rmCreateArea("wrinkle"+i);
      rmSetAreaSize(elevID, rmAreaTilesToFraction(1200), rmAreaTilesToFraction(1220));
      rmSetAreaLocation(elevID, rmRandFloat(0.0, 1.0), rmRandFloat(0.0, 1.0));
      rmSetAreaWarnFailure(elevID, false);
      rmSetAreaBaseHeight(elevID, rmRandFloat(4.0, 5.0));
      rmSetAreaMinBlobs(elevID, 6);
      rmSetAreaMaxBlobs(elevID, 10);
      rmSetAreaMinBlobDistance(elevID, 16.0);
      rmSetAreaMaxBlobDistance(elevID, 200.0);
      rmSetAreaCoherence(elevID, 0.0);
      rmAddAreaConstraint(elevID, shortCliffConstraint);
      rmAddAreaConstraint(elevID, avoidBuildings);
      rmAddAreaConstraint(elevID, playerConstraint); 
      if(rmBuildArea(elevID)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
   }

   rmPlaceObjectDefPerPlayer(startingSettlementID, true);

   id=rmAddFairLoc("Settlement", true, true, 90, 1200, 90, 30);
   rmAddFairLocConstraint(id, avoidImpassableLand);
   rmAddFairLocConstraint(id, farStartingSettleConstraint);
   rmAddFairLocConstraint(id, playerConstraint);
   rmAddFairLocConstraint(id, settleConstraint);
   rmAddFairLocConstraint(id, shortAvoidSettlement);
   rmAddFairLocConstraint(id, farAvoidSettlement);

   if(rmPlaceFairLocs())
   {
      id=rmCreateObjectDef("far settlement2");
      rmAddObjectDefItem(id, "Settlement", 3, 100.0);
      rmAddFairLocConstraint(id, avoidImpassableLand);
      rmAddFairLocConstraint(id, farStartingSettleConstraint);
      rmAddFairLocConstraint(id, playerConstraint);
      rmAddFairLocConstraint(id, settleConstraint);
      rmAddFairLocConstraint(id, shortAvoidSettlement);
      rmAddFairLocConstraint(id, farAvoidSettlement);



      for(i=1; <cNumberPlayers)
      {
         for(j=0; <rmGetNumberFairLocs(i))
            rmPlaceObjectDefAtLoc(id, i, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j), 1);
      }
   }
   // Towers.
   rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

 

   // Straggler trees.
   rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(12, 8));
   
   // Text
   rmSetStatusText("",0.50);


   // Gold
   rmPlaceObjectDefPerPlayer(startingGoldID, false, 2);

   // Goats
   rmPlaceObjectDefPerPlayer(closegoatsID, true);

   // Chickens or berries.
   for(i=1; <cNumberPlayers)
   {
      if(rmRandFloat(0.0, 1.0)<0.5)
         rmPlaceObjectDefAtLoc(closeChickensID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      else
         rmPlaceObjectDefAtLoc(closeBerriesID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
   }

   // Boar.
   rmPlaceObjectDefPerPlayer(closeBoarID, false);

   // Medium things....

   // goats
   rmPlaceObjectDefPerPlayer(mediumGoatsID, false, rmRandInt(1, 2));
   
   // Far things.
   
   // Gold.
   rmPlaceObjectDefInArea(farGoldID, 0, goldArea, cNumberNonGaiaPlayers*rmRandInt(3, 6));

   // Relics
   rmPlaceObjectDefPerPlayer(relicID, false);
   
   // Text
   rmSetStatusText("",0.60);

   // goats
   rmPlaceObjectDefPerPlayer(farGoatsID, false, rmRandInt(2, 2));

   // Bonus huntable.
   rmPlaceObjectDefAtLoc(bonusHuntableID, 0, 0.5, 0.5, cNumberNonGaiaPlayers);

   // Berries.
   rmPlaceObjectDefAtLoc(farBerriesID, 0, 0.5, 0.5, cNumberPlayers);

   // Predators
   rmPlaceObjectDefPerPlayer(farPredatorID, false, 1);

   // Random trees.
   rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers); 

   // Text
   rmSetStatusText("",0.70);

   // Forest.
   int classForest=rmDefineClass("forest");
   int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 15.0);
   int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 25.0);
   int forestCount=40*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <forestCount)
   {
      int forestID=rmCreateArea("forest"+i);
      rmSetAreaSize(forestID, rmAreaTilesToFraction(85), rmAreaTilesToFraction(110));
      rmSetAreaWarnFailure(forestID, false);
      if(rmRandFloat(0.0, 1.0)<0.25)
         rmSetAreaForestType(forestID, "mixed oak forest");
      else
         rmSetAreaForestType(forestID, "mixed oak forest");
      rmAddAreaConstraint(forestID, forestSettleConstraint);
      rmAddAreaConstraint(forestID, forestConstraint);
      rmAddAreaToClass(forestID, classForest);
      
      rmSetAreaMinBlobs(forestID, 65);
      rmSetAreaMaxBlobs(forestID, 95);
      rmSetAreaMinBlobDistance(forestID, 5.0);
      rmSetAreaMaxBlobDistance(forestID, 1000.0);
      rmSetAreaCoherence(forestID, 0.2);

      if(rmBuildArea(forestID)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
   }



   // Text
   rmSetStatusText("",0.80);

   // Grass
   int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);

   // Text
   rmSetStatusText("",0.90);


 
   int kelp4ID=rmCreateObjectDef("mist");
   rmAddObjectDefItem(kelp4ID, "mist", 1, 0.0);
   rmSetObjectDefMinDistance(kelp4ID, 0.0);
   rmSetObjectDefMaxDistance(kelp4ID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(kelp4ID, avoidAll);
   rmAddObjectDefConstraint(kelp4ID, avoidImpassableLand);
   rmAddObjectDefConstraint(kelp4ID, mistCenterConstraint);
   rmPlaceObjectDefAtLoc(kelp4ID, 0, 0.5, 0.5, 10*cNumberNonGaiaPlayers); 

   int kelp9ID=rmCreateObjectDef("damned this place is wet");
   rmAddObjectDefItem(kelp9ID, "rain", 1, 0.0);
   rmSetObjectDefMinDistance(kelp9ID, 0.0);
   rmSetObjectDefMaxDistance(kelp9ID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(kelp9ID, avoidAll);
   rmAddObjectDefConstraint(kelp9ID, avoidImpassableLand);
   rmAddObjectDefConstraint(kelp9ID, mistCenterConstraint);
   rmPlaceObjectDefAtLoc(kelp9ID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers); 

   int kelp7ID=rmCreateObjectDef("Blowing Leaves");
   rmAddObjectDefItem(kelp7ID, "tamarisk tree", 1, 0.0);
   rmAddObjectDefItem(kelp7ID, "Blowing Leaves", 2, 2.0);
   rmSetObjectDefMinDistance(kelp7ID, 0.0);
   rmSetObjectDefMaxDistance(kelp7ID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(kelp7ID, avoidAll);
   rmAddObjectDefConstraint(kelp7ID, avoidImpassableLand);
   rmAddObjectDefConstraint(kelp7ID, tamariskCenterConstraint);
   rmPlaceObjectDefAtLoc(kelp7ID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);

   // Nasty dudes
   int BlowingLeavesID=rmCreateObjectDef("Tolkien would be proud");
   rmAddObjectDefItem(BlowingLeavesID, "troll", 1, 0.0);
   rmSetObjectDefMinDistance(BlowingLeavesID, 0.0);
   rmSetObjectDefMaxDistance(BlowingLeavesID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(BlowingLeavesID, avoidImpassableLand);
   rmAddObjectDefConstraint(BlowingLeavesID, mistCenterConstraint);
   rmPlaceObjectDefAtLoc(BlowingLeavesID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);

   int kelp15ID=rmCreateObjectDef("bad boy");
   rmAddObjectDefItem(kelp15ID, "mountain giant", 1, 0.0);
   rmSetObjectDefMinDistance(kelp15ID, 0.0);
   rmSetObjectDefMaxDistance(kelp15ID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(kelp15ID, avoidAll);
   rmAddObjectDefConstraint(kelp15ID, avoidImpassableLand);
   rmAddObjectDefConstraint(kelp15ID, mistCenterConstraint);
   rmPlaceObjectDefAtLoc(kelp15ID, 0, 0.5, 0.5, 2*cNumberNonGaiaPlayers);

   int kelp5ID=rmCreateObjectDef("birdie");
   rmAddObjectDefItem(kelp5ID, "hawk", 1, 0.0);
   rmSetObjectDefMinDistance(kelp5ID, 0.0);
   rmSetObjectDefMaxDistance(kelp5ID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(kelp5ID, avoidAll);
   rmAddObjectDefConstraint(kelp5ID, avoidImpassableLand);
   rmPlaceObjectDefAtLoc(kelp5ID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);

   int kelp41ID=rmCreateObjectDef("mist");
   rmAddObjectDefItem(kelp41ID, "mist", 1, 0.0);
   rmSetObjectDefMinDistance(kelp41ID, 0.0);
   rmSetObjectDefMaxDistance(kelp41ID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(kelp41ID, avoidAll);
   rmAddObjectDefConstraint(kelp41ID, avoidImpassableLand);
   rmAddObjectDefConstraint(kelp41ID, mistCenterConstraint);
   rmPlaceObjectDefAtLoc(kelp41ID, 0, 0.5, 0.5, 25*cNumberNonGaiaPlayers); 

   int rockID2=rmCreateObjectDef("rock group");
   rmAddObjectDefItem(rockID2, "rock limestone small", 1, 1.0);
   rmAddObjectDefItem(rockID2, "rock limestone sprite", 3, 2.0);
   rmSetObjectDefMinDistance(rockID2, 0.0);
   rmSetObjectDefMaxDistance(rockID2, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(rockID2, avoidAll);
   rmAddObjectDefConstraint(rockID2, avoidImpassableLand);
   rmAddObjectDefConstraint(rockID2, patchConstraint);
   rmAddObjectDefConstraint(rockID2, playerConstraint);
   rmPlaceObjectDefAtLoc(rockID2, 0, 0.5, 0.5, 15*cNumberNonGaiaPlayers); 

   int kelp2ID=rmCreateObjectDef("bush");
   rmAddObjectDefItem(kelp2ID, "bush", 1, 2.0);
   rmAddObjectDefItem(kelp2ID, "grass", 2, 4.0);
   rmSetObjectDefMinDistance(kelp2ID, 0.0);
   rmSetObjectDefMaxDistance(kelp2ID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(kelp2ID, avoidAll);
   rmAddObjectDefConstraint(kelp2ID, avoidImpassableLand);
   rmPlaceObjectDefAtLoc(kelp2ID, 0, 0.5, 0.5, 15*cNumberNonGaiaPlayers);


   int kelp22ID=rmCreateObjectDef("xtra tree");
   rmAddObjectDefItem(kelp22ID, "oak", 1, 2.0);
   rmSetObjectDefMinDistance(kelp22ID, 0.0);
   rmSetObjectDefMaxDistance(kelp22ID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(kelp22ID, avoidAll);
   rmAddObjectDefConstraint(kelp22ID, avoidImpassableLand);
   rmPlaceObjectDefAtLoc(kelp22ID, 0, 0.5, 0.5, 15*cNumberNonGaiaPlayers);

   // Text


   // Text

   rmSetStatusText("",1.00); 
}  




